#ifdef IGNORE_ME
  #define ident "@(#) c_fixed_length_record.h (/sccs/ri_util/include/s.c_fixed_length_record.h) 1.3 02/24/99 14:20:44";
#endif


/*
 * file:  c_fixed_length_record.h
 *
 * description: declaration of c_fixed_length_record class
 *
 * revision list:
 *  Wed May  6 18:31:02 EDT 1998 KRB - created file
 *
 *
 *
 *
 */

/*

=head1 NAME 

c_fixed_length_record and c_fixed_length_field

=head1 SYNOPSIS

  #include <c_fixed_length_record.h>
  ...
  // the rest is too complicated for a synopsis, read on...

=head1 DESCRIPTION

These objects are for working with field length records.  They provide
a clean, concise, easy way for parsing and manipulating fixed length
strings of data.

The fixed length record object consists of an orderd array of fixed
length field objects.  Each of the field objects is responsible for
keeping track of it's name, it's data, the start position, and the
end position of the data it represents in the fixed record.

=head1 API REFERENCE

=head2 c_fixed_length_field::c_fixed_length_field( ostream& o = cout ) 

This is the default constructor for a field object.

=head2 c_fixed_length_field::c_fixed_length_field()

This is an initializing constructor.  With it, you can specify 
the following four parameters: the start position of the data, 
the end position of the data, the locical name of the field,
and the data for the field.

=head2 c_fixed_length_field::c_fixed_length_field( const c_fixed_length_field& that ) 

Copy constructor, copies an existing field object.

=head2 virtual c_fixed_length_field::~c_fixed_length_field(void)

Desctructor.

=head2  virtual int c_fixed_length_field::get_name( c_string& _name ) const

Sets _name to the name stored in the object.

=head2 virtual int c_fixed_length_field::get_value( c_string& _value ) const

Sets _value equal to the value stored in the object.

=head2 virtual int c_fixed_length_field::get_begin(void)     const

=head2 virtual int c_fixed_length_field::get_begin( int& b ) const

=head2 virtual int c_fixed_length_field::get_end(void)       const

=head2 virtual int c_fixed_length_field::get_end( int& e )   const

These functions are used for getting each of the respective member values 
from the field object.

=head2 virtual int c_fixed_length_field::set_name( const c_string& n )   

=head2 virtual int c_fixed_length_field::set_value( const c_string& v )  

=head2 virtual int c_fixed_length_field::set_begin( int b )              

=head2 virtual int c_fixed_length_field::set_end( int e )                

These functions are used for setting each of the respective member values 
in the field object.

=head2 virtual const c_fixed_length_field& c_fixed_length_field::operator=( const c_fixed_length_field& that )

This is an assignment operator for copying another field object.

=head2 virtual int c_fixed_length_field::operator==(const c_fixed_length_field&) const

This is an equality operator to test weather or not two fixed length
field objects are identical.

=head2 friend ostream& operator<<( ostream& o, const c_fixed_length_field& field )

This friend fucntion is provided for streaming capabilities.  Since the
internaly stored value information is character data, this provides a simple
way for having that inforamtion streamed to an ostream object.

=head2 void dump( void ) 

Used for debugging purposes.  It dumps the internal state of the object to
standard out.

=head1 BUGS

No known bugs.

=head1 AUTHORS

Kyle R. Burton mortis@voicenet.com

=cut

*/

#ifdef IGNORE_ME
#endif

#if !defined( _RC_FIXED_LENGTH_RECORD ) 
#define _RC_FIXED_LENGTH_RECORD 

#include <iostream.h>
#include <stdlib.h>

#include <c_string.h>
#include <c_timestamp.h>
#include <c_hash.h>

class c_fixed_length_field
{
public:
  c_fixed_length_field( ostream& o = cout ) 
    : begin(0)
    , end(0)
    , name()
    , value()
    , out(o)
  {
    // do nothing...
  };

  c_fixed_length_field( 
    unsigned int _begin, 
    unsigned int _end, 
    const char* _name,
    const char* _value = (const char*)NULL,
    ostream& o = cout
  )
    : begin(_begin)
    , end(_end)
    , name()
    , value()
    , out(o)
  {
    name = _name;
    if( value ) value = _value;
  };

  c_fixed_length_field(
    const char* _begin,
    const char* _end,
    const char* _name,
    const char* _value = (const char*)NULL,
    ostream& o = cout
  )
    : begin(0)
    , end(0)
    , name((const char*)NULL)
    , value((const char*)NULL)
    , out(o)
  {
    begin = (unsigned int)atoi(_begin);
    end   = (unsigned int)atoi(_end);

    name = _name;
    if( _value ) value = _value; 
  };

  c_fixed_length_field( const c_fixed_length_field& that ) 
    : begin(that.begin)
    , end(that.end)
    , name(that.name)
    , value(that.value)
    , out(that.out)
  {
    // do nothing else...
  };

  virtual ~c_fixed_length_field(void) {
    // do nothing...
  };

  virtual int get_name( c_string& _name ) const {
    _name = name;
    return TRUE;
  };

  virtual int get_value( c_string& _value ) const {
    _value = value;
    return TRUE;
  };

  virtual int get_begin(void)     const { return begin; };
  virtual int get_begin( int& b ) const { b = begin; return begin; };
  virtual int get_end(void)       const { return end; };
  virtual int get_end( int& e )   const { e = end; return end; };

  virtual int set_name( const c_string& n )   { name = n; return TRUE; };
  virtual int set_value( const c_string& v )  { value = v; return TRUE; };
  virtual int set_begin( int b )              { begin = b; return TRUE; };
  virtual int set_end( int e )                { end = e;   return TRUE; };

  virtual const c_fixed_length_field& operator=( const c_fixed_length_field& that ) {
    begin = that.begin;
    end = that.end;
    name = that.name;
    value = that.value;

    return *this;
  };

  virtual int operator==( const c_fixed_length_field& that ) const {
    if( begin != that.begin || end != that.end ) {
      return FALSE;
    }

    if( name != that.name ) {
      return FALSE;
    }
    
    // should we check the value???
    if( value != that.value ) {
      return FALSE;
    }

    return TRUE;
  };

  friend ostream& operator<<( ostream& o, const c_fixed_length_field& field ) {
    o << NOW << COUT_FILE << "[c_fixed_length_field] Dump:" << endl;
    o << NOW << COUT_FILE << "  name   = " << field.name << endl;
    o << NOW << COUT_FILE << "  value  = " << field.value << endl;
    o << NOW << COUT_FILE << "  begin  = " << field.begin << endl;
    o << NOW << COUT_FILE << "  end    = " << field.end << endl;

    return o;
  };

  void dump( void ) {
    operator<<( out, *this );
  };

private:
  c_string  name;
  c_string  value;
  unsigned int  begin;
  unsigned int  end;
  ostream&  out;
};

class c_fixed_length_record
{
public:
  c_fixed_length_record( ostream& o = cout );
  c_fixed_length_record( const c_fixed_length_record& that );
  virtual ~c_fixed_length_record(void);

  // adds a new field to the list
  virtual int add_field( const c_fixed_length_field& field );
  virtual int add_field( const char* name );
  virtual int add_field( const char* name, const char* value );
  virtual int add_field( 
    unsigned int _begin, 
    unsigned int _end, 
    const char* name, 
    const char* value = (const char*)NULL
  );

  virtual int add_field( 
    const char* _begin, 
    const char* _end, 
    const char* name, 
    const char* value = (const char*)NULL
  );

  // assignment operator for other fixed length record objects...
  virtual const c_fixed_length_record& operator=( const c_fixed_length_record& right );
  virtual int operator==( const c_fixed_length_record& right ) const;

  virtual const c_fixed_length_record& operator+=( const c_fixed_length_record& right );
  friend const c_fixed_length_record operator+( 
    const c_fixed_length_record& left,
    const c_fixed_length_record& right
  );

  virtual int get_value( const char* field, char* value ) const ;
  virtual int get_value( const char* field, c_string& value ) const ;
  virtual int get_value( const c_string& field, c_string& value ) const ;

  virtual int get_field( const char* field_name, c_fixed_length_field& field ) const;

  virtual int remove_field( const char* field_name );

  virtual int parse( const char* s );

  friend ostream& operator<<( ostream& o, const c_fixed_length_record& record );
  void dump(void) const;

  void clear(void);
  void clear_fields(void);

  int num_fields( void ) const { return fields.get_count(); };

  int get_field_names( c_collection<c_string>& names ) const ;
private:
  c_hash<c_string,c_fixed_length_field>    fields;
  ostream&                                  out;
};


#endif
