#ifdef IGNORE_ME
  #define ident "@(#) c_multi_tree.h (/sccs/ri_util/include/s.c_multi_tree.h) 1.12 02/24/99 14:20:45";
#endif


/*  c_multi_tree.h
    Copyright 1998 Kyle R. Burton

c_multi_tree.h is free software; you can redistribute it and/or modify
it under the tearms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

c_multi_tree.h is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with c_multi_tree.h see the file COPYING.  If not, write to
the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

*/

/*

=head1 NAME

c_multi_tree - multi node tree object

=head1 SYNOPSIS

  #include <c_multi_tree.h>
  ...
  // the rest is too complex for a snyopsis, read on

=head1 DESCRIPTION

c_mulit_tree is a tree object that stores key/value pairs
where key uniqueness is not enforced, and where a node
in the tree can have more than one child node.

The tree objectis just a place holder, while the manipulator
[handle] object is used to perform all manipulation of the tree,
including node creation, deletion, and all traversal
functions.

The manipulator object relates to the tree in two distinct
ways.  First, it associates itself with a particular node
in the tree.  Second, it associates itself with that node's
children.  These relationships will be clearer as you read
and understand the member functions of the manipulator object.

It was originaly creatd to model the relationship between
records from a relational database, where you wanted
to store the resutls of a set of queries in a heirarchical
structure.  This does not prevent the tree object from being
used to model other heirarchical relationships, it's just
the one that was the necessity that motherd the tree's 
invention.

It is also possible to think of the multi tree object
as a multi dimensional array where both the size of
the arrays, and the number of dimensions are arbritrary,
and dynamicly growable at runtime!

For object stored in the tree (both keys and values), they must 
supply the following member funcitons:

  a default constructor        T::T()
  a copy constructor           T::T(const T&)
  an assignment operator       T::operator=(const T&)
  an equality operator         T::operator==(const T&)


=head1 API REFERENCE - c_multi_tree

=head2 c_multi_tree( void ) 

Default constructor.

=head2 c_multi_tree( const c_multi_tree_node<KEY,VALUE>& that ) 

This is a copy constructor, used internaly by the multi tree 
objects.  

=head2  c_multi_tree( const c_multi_tree<KEY,VALUE>& that ) 

Copy constrcutor to copy a tree object.

=head2 virtual ~c_multi_tree( void )

Destructor for the tree.

=head1 API REFERENCE c_multi_tree_manipulator

=head2 c_multi_tree_manipulator( c_multi_tree<KEY,VALUE>& _tree ) 

This is the default constructor for the manipulator [handle]
object.  You must pass a tree object into the constrcutor.

=head2 c_multi_tree_manipulator( const c_multi_tree_manipulator<KEY,VALUE>& that ) 

This is a copy constructor for the handle object.

=head2  virtual ~c_multi_tree_manipulator( void )

Destructor member.  Deletes the manipulator object, but 
not the tree object.  Destrucing a handle has no effect 
on a tree.

=head2 virtual int top( void )

Positions the handle to point at the top of the assocaited
c_multi_tree object's tree.

=head2 virtual int first(void)

This sets the child pointer to the first child node of the current
node the handle is pointing at.
Returns true on success, false on failure.

=head2 virtual int next(void)

Moves the child pointer to the next child node of the current
node.
Returns true on success, false on failure.

=head2 virtual int prev(void) {

Moves the child pointer to the previous child node of the current
node.
Returns true on success, false on failure.

=head2 virtual int last(void)

This sets the child pointer to the last child node of the current
node the handle is pointing at.
Returns true on success, false on failure.

=head2 virtual int position( int pos = -1 )

Sets the child pointer to the child node at the given
position.

=head2 virtual int get_child_position( void )

This function returns the index of the child node
that the child pointer currently points at.
Returns index on success, -1 on failure.

=head2 virtual int down( int pos = -1 )

Traverses down in the tree.  If -1 is passed for pos, it
will attempt to traverse down into the node that the
current child point to.  If an index is passed for pos,
it will atempt to set the child pointer to point at that
node first, and then traverse down. 

This traversal causes the current child pointer to become 
invalid.

Returns true on success, false on failure.

=head2 virtual int up( void )

Traverses up in the tree.  Invalidates the current child.
Returns true on success, false on failure.

=head2 virtual int new_parent( void )

This function allows the user to create a new parent node
if and only if the current node is the top of the tree.
It does not traverse to the newly created node, just adds
it to the tree.
Returns true on success, false on failure.

=head2 virtual int get_key( KEY& k ) const

Sets k to the key for the current node.
Returns true on success, false on failure.

=head2 virtual int set_key( const KEY& k )

Sets the current node's key to k.
Returns true on success, false on failure.

=head2 virtual int get_value( VALUE& v ) const

Sets v to the value for the current node.
Returns true on success, false on failure.

=head2 virtual int set_value( const VALUE& v ) const

Sets the current node's value to v.
Returns true on success, false on failure.

=head2 virtual int get_data( KEY& k, VALUE& v ) const

Gets both the key and value for the current node.
Returns true on success, false on failure.

=head2 virtual int set_data( KEY& k, VALUE& v ) const

Sets both the key and value for the current node.
Returns true on success, false on failure.

=head2 virtual int get_child_key( KEY& k, int pos = -1 )

Gets the key value for the node that the current 
child pointer references.
Returns true on success, false on failure.

=head2 virtual int set_child_key( const KEY& k, int pos = -1 )

Sets the key value for the node that the current 
child pointer references.
Returns true on success, false on failure.

=head2 virtual int get_child_key( KEY& k, int pos = -1 )

Sets k to the key for the current child.
Returns true on success, false on failure.

=head2 virtual int set_child_key( const KEY& k, int pos = -1 )

Sets key for the current child to k.
Returns true on success, false on failure.

=head2 virtual int get_child_data( KEY& k, VALUE& v, int pos = -1 )

Gets both the value, and the key for the current child.
Returns true on success, false on failure.

=head2 virtual int set_child_data( KEY& k, VALUE& v, int pos = -1 )

Sets both the value, and the key for the current child.
Returns true on success, false on failure.

=head2 virtual int delete_child( int pos = -1 )

Removes a child node from the current node.  Removes
the current node, or the speicfied node if a non-negative
number is passed for pos.  If the child has children of it's
own, pruning this tree, destroys those trees also.
Returns true on success, false on failure.

=head2 virtual int delete_children( void )

Removes all child nodes from the current node.
Returns true on success, false on failure.

=head2 virtual int new_child( int pos = -1 )

Creates a new child node at the end of the current
list of child nodes, or just before the given position.
Returns true on success, false on failure.

=head2 virtual int new_child( const KEY& k, const VALUE& v, int pos = -1 )

Creates a new child node at the end of the current
list of child nodes, or just before the given position, and 
initializes it with the supplied data.
Returns true on success, false on failure.

=head2 virtual int num_children( void ) const

Returns the number of children for the current node.
Returns number of children on success, -1 on failure.

=head2 virtual int childs_num_children( int pos = -1 ) const {

Returns the number of children for the child node that the
current child pointer references.
Returns number of children on success, -1 on failure.

=head2 virtual int copy( const c_multi_tree_manipulator<KEY,VALUE>& right )

Makes the current node a copy of the current node referenced by the
given handle.
Returns true on success, false on failure.

=head2 virtual int find_child_pos_by_key( const KEY& key )

Useful for locating the first child that has a key that matches
the given key.
Returns position on success, -1 on failure.

=head2 virtual int find_child_pos_by_value( const VALUE& value ) {

Looks for the first child node that contains a value that 
matches the given value.
Returns position on success, -1 on failure.

=head2 virtual const c_multi_tree_manipulator<KEY,VALUE>& operator= const c_multi_tree_manipulator<KEY,VALUE>& right )

Assignment operator for manpulator objects.  Copies the data in the
other tree handle.  After assignment, both trees manipulators will 
reference the same spot in the origianl tree.  This function does
not copy the actual tree, or any of it's data.
Returns true on success, false on failure.

=head2 virtual const c_multi_tree_manipulator<KEY,VALUE>& operator= c_multi_tree<KEY,VALUE>& right )

Assignment operator for manpulator objects.  Resets the handle
object just as if it was constructed with the provided tree.
This function does not copy the actual tree, or any of it's data.
Returns true on success, false on failure.

=head2 virtual int operator==( const c_multi_tree_manipulator<KEY,VALUE>& right )

Equality operator used for comparing two manipulator objects.
Returns true on success, otherwise returns false.

=head1 BUGS

No known bugs.

=head1 AUTHORS

Kyle R. Burton  mortis@voicenet.com
Yan Nabedrik    ynabedri@rosenbluth.com

=head1 SEE ALSO

Books on data structures.  Try Kunth for starters.

*/


/*
 * file: c_multi_tree.h
 *
 * description:  declaration of a multi node tree object.
 *  initial design will use linked lists...
 *
 * refrences: 
 *   Algorithms in C++ 
 *      Robert Sedgwick
 *      Addison Wesley 1992
 *      ISBN 0201510596
 *   The Art of Computer Programming  Volume 1 Fundamental Algorithms
 *     third edition, Donald E. Knuth
 *
 * notes:  as compared to the collection objects, the tree object
 *  presents a problem with traversal.  The collection objects provide
 *  a high degree of safeness -- they don't allow you to touch the
 *  internal elements of the collection during traversal.  This multi-
 *  node tree is a bit different -- during traversal, each node is data
 *  _and_ another tree.  So providing a copy for you to play with is 
 *  inefficient in the extreme.  I'm not sure if it's necessary to
 *  give access to the node during traversal.
 *
 *  We have not yet decided how we are going to handle traversal.  This
 *  raises issues of memory management, who's resonsible for what, and 
 *  what happens to refrences/pointers that you have when the tree goes
 *  away...
 *
 *  Q: Why are these classes being entirely implemented in a header file?
 *     Wouldn't they be easier to code/understand/etc... if they were broken
 *     down into header, and source files?
 *  
 *  A: Yes.  These reasons are very strong arguments for doing it that way 
 *     (seperate files), but they are not the one that I allowed to make the
 *     final decision.  Some C++ compilers I've come across can't correctly
 *     generate the template specializations if the files are broken down into
 *     source and header files (specialization is when the compiler takes
 *     a template/genralized class, and creates a version that is specific
 *     to the class which youre using it for -- if you do this: c_stack<int>, 
 *     and c_stack<char>, the compiler has to generate two specializations
 *     one for c_stack<int>, and one for c_stack<char>).  Since one of my
 *     strongest design criteria was portability, I've opted to do the implmentation
 *     completely in the header file.
 *  
 *
 * proposed approaches?:
 * 1 use a recursive data structure - i.e. the nodes are a tree object
 *   issues:
 *     - the user gets to manipulate the tree nodes
 *       this is dangerous due to memory management?
 *     - it's sloppy -- the user has to know how to traverse the tree
 *       and how the nodes are structured.  if this stuff changes, your
 *       traversal code might break.
 *     - who's responsible for memory management?  if the user is direcly
 *       touching the nodes in the tree, they can do funny stuff -- like
 *       delete parts of the tree without cleaning up
 *
 * 2 use a tree handle object
 *     use an object for the nodes [user doens't get to touch]
 *     use an object for holding the tree [user gets to have, but do anything with]
 *     use an object for manipulating the tree [user gets to have/use]
 *   issues:
 *     - if done correctly, this can give us the brick wall between the data/code
 *       that deals with the tree, and code that uses the tree, that we're looking
 *       for
 *     - it should provide the same level of saftey as the collection objects.
 *     - ???
 *
 *
 * revision list: 
 *    Wed Aug 12 10:11:38 EDT 1998 KRB - created the file
 *    Thu Aug 13 08:50:16 EDT 1998 KRB - setteled on a design -- a multi tree
 *      object (which is almost useless), and a multi tree manipulator object,
 *      which has to be used to do anything at all with the tree.  this includes
 *      inserts, removals, modifcation, and querying of a node.  This was basicly
 *      choice number 2, only I wasn't considering using the manipulator for all
 *      the accesses to the tree's structure...
 *
 *      This design is a derivative of #2 above -- I also modified the original
 *      description of #2 to fit this more closely.
 *    Fri Aug 14 13:49:24 EDT 1998 KRB - ok, now there's an issue with weather
 *      or not to use pointers, or refences in the nodes themselves.  I would
 *      really like to use refrences, but I'm not sure that's going to work -- the
 *      refrence variable may need to change to point at a new node dynamicly without
 *      destructing, and then recreating the object.
 *    Mon Aug 17 17:10:31 EDT 1998 KRB - initial compile withouth any warnings, or
 *      errors!  of course this doens't mean diddly works in the code, but at least you
 *      can construct and destruct the top level objects!  oh happy day!  I'm sure 
 *      the runtime behavior of the code at this point will rain on my parade, but at least
 *      I'm satisfied enough to quit for the day...
 *    Tue Aug 18 10:15:37 EDT 1998 KRB/YN - Yan and I were going over the interface to
 *      the c_multi_tree_manipulator object, and came up with a few revisions -- new
 *      to the world are:  num_children(void), delete_children(void), and the child
 *      data access functions -> get_child_key(), get_child_value(), etc...
 *    Tue Aug 18 17:07:05 EDT 1998 KRB - this puppy's ready for unit testing, it
 *      compiles cleanly, and as far as I know all the member functions are
 *      implemented correctly (this is an outright lie at best, as I really feel
 *      that 80% of the code is buggy, but who's counting...)
 *
 */


#if !defined ( _C_MULTI_TREE )
#define _C_MULTI_TREE

#include <stdinc.h>
#include <c_timestamp.h>
#include <c_collection.h>
#include <c_list.h>

//
// this forward declarations are necessary for the definition of both the 
// tree node, and the tree itself...(so these classes can be declared as
// friends)
//
template<class KEY,class VALUE> class c_multi_tree_manipulator;
template<class KEY,class VALUE> class c_multi_tree;

/*******************************************************************************
 *
 * tree node structure (class actually)
 *
 * contains:
 *  key
 *  value
 *  parent refrence
 *  list of children
 *
 * This object's only purpose in life is to be manipulated by the tree
 * manipulator object.
 *
 *
 */
template<class KEY,class VALUE>
class c_multi_tree_node {
  // these are protected so only friends can construct the node objects...
protected:
  ////////////////////////////////////////////////////////////
  // default constructor, takes no arguments
  //
  c_multi_tree_node( void )
    : key()
    , value() 
    , p_parent( NULL )
    , children() {
    ; // do nothing
  };

  ////////////////////////////////////////////////////////////
  // initializing constructor, takes a single key, and a 
  // single value
  //
  c_multi_tree_node( const KEY& k, const VALUE& v )
    : key(k)
    , value(v)
    , p_parent( NULL ) 
    , children() {
    ; // do nothing
  };

  ////////////////////////////////////////////////////////////
  // copy constructor
  //
  c_multi_tree_node( const c_multi_tree_node<KEY,VALUE>& that )
    : key()
    , value()
    , p_parent( NULL ) 
    , children() {
    *this = that;
  };

public:
  ////////////////////////////////////////////////////////////
  // destructor
  //
  virtual ~c_multi_tree_node() {
    // step through the children, and delete each one...
    c_multi_tree_node<KEY,VALUE>* tmp;
    while( children.remove_head(tmp) ) {
      delete tmp;
    }
  };

protected:
  ///////////////////////////////////////////////////
  // assignment operator
  //
  // if you're looking at this function because of
  // a bus error, or a sigsev, or some such animal,
  // please check the depth of your tree -- this is
  // a recursive funciton, if you're tree is _really_
  // deep, then you probably blew away  the stack...
  // increace the stack size, or use a smaller 
  // tree smart guy
  //
  virtual const c_multi_tree_node<KEY,VALUE>& 
    operator=( const c_multi_tree_node<KEY,VALUE>& right ) {
    // delete all our children, and clear ourselves...
    clear_all();

    key   = right.key;
    value = right.value;

    // copy the other tree...
    c_multi_tree_node<KEY,VALUE>* p_new, *tmp;
    c_collection_iter<c_multi_tree_node<KEY,VALUE>*>* iter;
    iter = right.children.prepare_iter();
    while( right.children.next(iter,tmp) ) {
      p_new = new c_multi_tree_node<KEY,VALUE>(*tmp);
      p_new->p_parent = this;
      children.add(p_new);
    }
    right.children.complete_iter(iter);

    return *this;
  };

  // equality operator...this should just check key vs key?
  virtual int operator==( const c_multi_tree_node<KEY,VALUE>& right ) const {
    return key == right.key;
  };

  // allow c_multi_tree_manipulator to tinker with our internals
  friend class c_multi_tree_manipulator<KEY,VALUE>;
  friend class c_multi_tree<KEY,VALUE>;

private:
  // remove all of our child nodes, and blank out this node.
  int clear_all( void ) {
    key = KEY();
    value = VALUE();

    c_multi_tree_node* tmp;
    while( children.remove_head(tmp) ) {
      delete tmp;
    }

    return( FALSE );
  };

protected:
  KEY                                   key;
  VALUE                                 value;
  c_multi_tree_node<KEY,VALUE>*         p_parent;
  c_list<c_multi_tree_node<KEY,VALUE>*> children;
};

/*******************************************************************************
 *
 * multi tree object
 *
 * This object's only reason for being is to store the head of the tree!
 * Thats it!  No magic here.  Of course, this object is pretty useless, except
 * as a handle to the tree.
 *
 *
 *
 */
template<class KEY,class VALUE>
class c_multi_tree {
public:
  c_multi_tree( void ) 
    : head(NULL) {
    head = new c_multi_tree_node<KEY,VALUE>();
  };

  c_multi_tree( const c_multi_tree_node<KEY,VALUE>& that ) 
    : head(NULL) {
    // copy the other tree...
    head = new c_multi_tree_node<KEY,VALUE>();
    copy_tree( head, that );
  };

  c_multi_tree( const c_multi_tree<KEY,VALUE>& that ) 
    : head(NULL) {
    // copy the other tree...
    head = new c_multi_tree_node<KEY,VALUE>();
    copy_tree( head, that.head );
  };

  virtual ~c_multi_tree( void ) {
    delete head;
  };

  friend class c_multi_tree_manipulator<KEY,VALUE>;
  friend class c_multi_tree_node<KEY,VALUE>;

protected:
  c_multi_tree_node<KEY,VALUE>  *head;
};

/*******************************************************************************
 *
 * multi tree manipulator
 *
 * This is how it's done.  This object is your window into the tree, you have
 * to use this object to do everything short of constructing the multi tree
 * object.
 *
 *
 * Thu Aug 13 15:58:59 EDT 1998 KRB - I'm still unsetteled about this interface
 * I'm not sure how I want to name the taxonomy of member functions for data
 * access or traversal, or even their interfaces...what's below should be looked
 * at as notes at best, pseudo code at worst.
 *
 *
 */
template<class KEY,class VALUE>
class c_multi_tree_manipulator {
public:
  c_multi_tree_manipulator( c_multi_tree<KEY,VALUE>& _tree ) 
    : tree(&_tree) 
    , p_node(_tree.head) 
    , p_child(NULL) 
    , child_pos(-1) {
   ; // do nothing
  };

  c_multi_tree_manipulator( const c_multi_tree_manipulator<KEY,VALUE>& that ) 
    : tree(that.tree) 
    , p_node(that.p_node) 
    , p_child(that.p_child)
    , child_pos(that.child_pos) {
    ; // do nothing
  };

  virtual ~c_multi_tree_manipulator( void ) {
    // definately don't deltete the tree, or the node pointer (or what's
    // at the pointer)...
    p_node  = NULL; // is there a reason for this?
    p_child = NULL; // is there a reason for this?
  };

  // jumps to the top of the tree...
  // think of this like hitting the reset button
  virtual int top( void ) {
    p_node = tree->head;
    p_child = NULL;
    child_pos = -1;

    return TRUE;
  };

  //
  // child traversal for data purposes...
  // these functions move the p_child pointer
  //

  // traverse across?
  // note:  (see also position()) I feel these should be const, but they can't
  //        because they'll modify p_child...
  virtual int first(void) {
    if( p_node->children.get_count() < 1 ) {
      return FALSE;
    }

    child_pos = 0;
    p_node->children.get_at(child_pos,p_child);
    return TRUE;
  };

  virtual int next(void) {
    ++child_pos;
    if( p_node->children.get_count() <= child_pos ) {
      child_pos = -1;
      return FALSE;  // invalid position, past the end of the list
    }

    p_node->children.get_at(child_pos,p_child);
    return TRUE;
  };

  virtual int prev(void) {
    --child_pos;
    if( child_pos < 0 || p_node->children.get_count() < 1 ) {
      child_pos = -1;
      return FALSE;
    }

    p_node->children.get_at(child_pos,p_child);
    return TRUE;
  };

  virtual int last(void) {
    if( p_node->children.get_count() < 1 ) {
      return FALSE;
    }

    child_pos = p_node->children.get_count() - 1;
    p_node->children.get_at(child_pos,p_child);
    return TRUE;
  };

  // move to a specific child node specified by it's index in the list.
  // note:  this will fail if pos is past the end of the list...
  // note:  somehow I feel this function should be const, but it can't
  //        or you can't modify stuff like p_node, or p_child...
  virtual int position( int pos = -1 ) {
    // this is reposition to no child pointer...kind of resets the 
    // manipulator for this level of the tree
    if( pos < 0 ) {
      p_child = NULL;
      return TRUE;
    }

    if( pos >= p_node->children.get_count() ) {
      return FALSE;
    }

    child_pos = pos;
    return p_node->children.get_at( pos, p_child );
  };


  // PLEASE NOTE THAT FOR THIS FUNCTION A RETURN VALUE
  // OF ZERO IS VALID, AS INDEXES START FROM 0
  // yes, it is _your_ falut is you treat zero as an error...
  // -1 will be returned if there is no current child...
  virtual int get_child_position( void ) {
    return child_pos;
  };

  // note:  again, I feel that this should be const, but it can't
  virtual int down( int pos = -1 ) {
    if( pos < 0 ) {
      if( NULL == p_child ) {
        return FALSE;
      }
      p_node = p_child;
      child_pos = -1;
      p_child = NULL;
    }
    else {
      c_multi_tree_node<KEY,VALUE>* tmp;
      if( !p_node->children.get_at(pos,tmp) ) {
        return FALSE;
      }

      p_node = tmp;
      child_pos = -1;
      p_child = NULL;
    }

    return TRUE;
  };

  // note:  again, the constness problem!
  virtual int up( void ) {
    if( NULL == p_node->p_parent ) {
      return FALSE;
    }

    p_child = NULL;
    child_pos = -1;
    c_multi_tree_node<KEY,VALUE>* p_tmp = p_node->p_parent;
    p_node = p_tmp;

    return TRUE;
  };

  // if we're at the top of the tree, let the user creat a new
  // parent, with this node as it's child...
  // there's no precedence for this in nature!
  // 
  // this makes the creation of trees during parsing situations
  // much easier where you recursivly parse, creating the leaf
  // nodes first, and the parents that tie them back togeather.
  //
  // imagine tracing your family tree from you on back, instead
  // of from your oldest known ancestors forward.  You identify
  // yourself, and all your siblings/cousins/neices/nephews, then 
  // your parents, and all their siblings/cousins/neices/nephews,
  // and continue up the tree till you reach the beginning...
  //
  virtual int new_parent( void ) {
    if( NULL != p_node->p_parent ) {
      return FALSE;
    }

    p_node->p_parent = new c_multi_tree_node<KEY,VALUE>();
    p_node->p_parent->children.add(p_node);

    return TRUE;
  };

  // manipulate the data at the current node
  virtual int get_key( KEY& k ) const {
    k = p_node->key;
    return TRUE;
  };

  virtual int set_key( const KEY& k ) {
    p_node->key = k;
    return TRUE;
  };

  virtual int get_value( VALUE& v ) const {
    v = p_node->value;
    return TRUE;
  };

  virtual int set_value( const VALUE& v ) {
    p_node->value = v;
    return TRUE;
  };

  virtual int get_data( KEY& k, VALUE& v ) const {
    k = p_node->key;
    v = p_node->value;
    return TRUE;
  };

  virtual int set_data( const KEY& k, const VALUE& v ) {
    p_node->key = k;
    p_node->value = v;
    return TRUE;
  };

  // manipulate the data at the current child...
  virtual int get_child_key( KEY& k, int pos = -1 ) {
    if( pos < 0 ) {
      if( NULL == p_child ) {
        return FALSE;
      }

      k = p_child->key;
    }
    else {
      // get the current child...
      c_multi_tree_node<KEY,VALUE>* tmp;
      if( !p_node->children.get_at(pos,tmp) ) {
        return FALSE;
      }

      k = tmp->key;
    }

    return TRUE;
  };

  virtual int set_child_key( const KEY& k, int pos = -1 ) {
    if( pos < 0 ) {
      if( NULL == p_child ) {
        return FALSE;
      }

      p_child->key = k;
    }
    else {
      c_multi_tree_node<KEY,VALUE>* tmp;
      if( !p_node->children.get_at(pos,tmp) ) {
        return FALSE;
      }

      tmp->key = k;
    }

    return TRUE;
  };


  virtual int get_child_value( VALUE& v, int pos = -1 ) {
    if( pos < 0 ) {
      if( NULL == p_child ) {
        return FALSE;
      }

      v = p_child->value;
    }
    else {
      c_multi_tree_node<KEY,VALUE>* tmp;
      if( !p_node->children.get_at(pos,tmp) ) {
        return FALSE;
      }

      v = tmp->value;
    }

    return TRUE;
  };

  virtual int set_child_value( const VALUE& v, int pos = -1 ) {
    if( pos < 0 ) {
      if( NULL == p_child ) {
        return FALSE;
      }

      p_child->value = v;
    }
    else {
      c_multi_tree_node<KEY,VALUE>* tmp;
      if( !p_node->children.get_at(pos,tmp) ) {
        return FALSE;
      }

      tmp->value = v;
    }

    return TRUE;
  };

  virtual int get_child_data( KEY& k, VALUE& v, int pos = -1 ) {
    if( pos < 0 ) {
      if( NULL == p_child ) {
        return FALSE;
      }

      k = p_child->key;
      v = p_child->value;
    }
    else {
      c_multi_tree_node<KEY,VALUE> *tmp;
      if( !p_node->children.get_at(pos,tmp) ) {
        return FALSE;
      }

      k = tmp->key;
      v = tmp->value;
    }

    return TRUE;
  };

  virtual int set_child_data( const KEY& k, const VALUE& v, int pos = -1 ) {
    if( pos < 0 ) {
      if( NULL == p_child ) {
        return FALSE;
      }
      p_child->key   = k;
      p_child->value = v;
    }
    else {
      c_multi_tree_node<KEY,VALUE> *tmp;
      if( !p_node->children.get_at(pos,tmp) ) {
        return FALSE;
      }

      tmp->key   = k;
      tmp->value = v;
    }

    return TRUE;
  };


  // -1 signifies current child, otherwise, you specify the child's number...
  virtual int delete_child( int pos = -1 ) {
    c_multi_tree_node<KEY,VALUE>* tmp;

    if( pos < 0 ) {
      if( NULL == p_child ) {
        return FALSE;
      }

      if( !p_node->children.remove(child_pos,tmp) ) { // gak!
        return FALSE;
      }

      if( p_child != tmp ) { // double gak!
        return FALSE;
      }

      p_child = NULL;
      child_pos = -1;
    }
    else {
      if( !p_node->children.get_at(pos,tmp) ) {
        return FALSE;
      }

      if( child_pos == pos ) {
        child_pos = -1;
        p_child = NULL;
      }
    }

    delete tmp;
    return TRUE;
  };

  // delete all the children...
  virtual int delete_children( void ) {
    c_multi_tree_node<KEY,VALUE>* tmp;

    while( p_node->children.remove_head(tmp) ) {
      delete tmp;
    }

    child_pos = -1;
    p_child = NULL;

    return TRUE;
  };


  // create a new child node...
  // with default key/value
  virtual int new_child( int pos = -1 ) {
    c_multi_tree_node<KEY,VALUE>* tmp = new c_multi_tree_node<KEY,VALUE>();
    tmp->p_parent = p_node;

    if( pos < 0 ) {
      // append it to the end...
      return p_node->children.add(tmp);
    }
    else {
      // validate pos...
      if( pos >= p_node->children.get_count() ) {
        return FALSE;
      }

      return p_node->children.insert(pos,tmp);
    }

    return TRUE;
  };

  // with sepcified key/value
  virtual int new_child( const KEY& k, const VALUE& v, int pos = -1 ) {
    c_multi_tree_node<KEY,VALUE>* tmp = new c_multi_tree_node<KEY,VALUE>(k,v);
    tmp->p_parent = p_node;

    if( pos < 0 ) {
      // append it to the end...
      return p_node->children.add(tmp);
    }
    else {
      // validate pos...
      if( pos >= p_node->children.get_count() ) {
        return FALSE;
      }

      return p_node->children.insert(pos,tmp);
    }

    return TRUE;
  };

  virtual int num_children( void ) const {
    return p_node->children.get_count();
  };

  // remember 0 means zero children, -1 means failure...this is 
  // a departure from the normal way of doing things...
  //
  // BE SURE TO CHECK THE RETURN VALUE OF THIS FUNCTION FOR
  // THE VALUE -1, THAT'S HOW YOU TELL WEATHER OR NOT IT FAILED
  //
  // Get that?  Good, now go away and leave me be.
  //
  virtual int childs_num_children( int pos = -1 ) const {
    if( pos < 0 ) {
      if( NULL == p_child ) {
        return -1;
      }

      return p_child->children.get_count();
    }
    else {
      c_multi_tree_node<KEY,VALUE>* tmp;
      if( !p_node->children.get_at(pos,tmp) ) {
        return -1;
      }

      return tmp->children.get_count();
    }

    return -1;
  };

  virtual int copy( const c_multi_tree_manipulator<KEY,VALUE>& right ) {
    if( NULL == p_node || NULL == right.p_node ) {
      return FALSE;
    }

    (*p_node) = (*right.p_node);

    return TRUE;
  };


  virtual int find_child_pos_by_key( const KEY& key ) {
    int pos = -1;
    if( NULL == p_node ) {
      return pos;
    }

    c_multi_tree_node<KEY,VALUE>* tmp;
    int num_children = p_node->children.get_count();
    for( int i = 0; i < num_children; ++i ) {
      if( !p_node->children.get_at(i,tmp) ) {
        break;
      }

      if( key == tmp->key ) {
        pos = i;
        break;
      }
    }

    return pos;
  };

  virtual int find_child_pos_by_value( const VALUE& value ) {
    int pos = -1;
    if( NULL == p_node ) {
      return pos;
    }

    c_multi_tree_node<KEY,VALUE>* tmp;
    int num_children = p_node->children.get_count();
    for( int i = 0; i < num_children; ++i ) {
      if( !p_node->children.get_at(i,tmp) ) {
        break;
      }

      if( value == tmp->value ) {
        pos = i;
        break;
      }
    }

    return pos;
  };

  virtual const c_multi_tree_manipulator<KEY,VALUE>& operator=(
    const c_multi_tree_manipulator<KEY,VALUE>& right ) {
    tree      = right.tree;
    p_node    = right.p_node;
    p_child   = right.p_child;
    child_pos = right.child_pos;

    return *this;
  };

  virtual const c_multi_tree_manipulator<KEY,VALUE>& operator=(
      c_multi_tree<KEY,VALUE>& right ) {
    tree      = &right;
    p_node    = NULL;
    p_child   = NULL;
    child_pos = -1;

    return *this;
  };

  // this is realy kind of useless, it's only implemented for the
  // c_collection objects -- if you're compareing two manipulators,
  // you should probably take a good look at what your really trying
  // to do...
  virtual int operator==( const c_multi_tree_manipulator<KEY,VALUE>& right ) {
    return( tree      == right.tree
         && p_node    == right.p_node
         && p_child   == right.p_child
         && child_pos == right.child_pos );
  };

private:
  c_multi_tree<KEY,VALUE>*      tree;    // the top of the tree
  c_multi_tree_node<KEY,VALUE>* p_node;    // the current node
  c_multi_tree_node<KEY,VALUE>* p_child;   // the current child of p_node...
  int                           child_pos;
};



#endif // !defined( _C_MULTI_TREE )
