#ifdef IGNORE_ME
  #define ident "@(#) c_string.h (/sccs/ri_util/include/s.c_string.h) 1.4 02/24/99 15:03:49";
#endif

  /*  c_string.h
  Copyright 1997 Kyle R. Burton

This file is part of c_string.h, an attempt at a win32 equivalent for
the system log daemon available under unix.

c_string.h is free software; you can redistribute it and/or modify
it under the tearms of the GNU General Public License as published by 
the Free Software Foundation; either version 2, or (at your option)
any later version.

c_string.h is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with c_string.h; see the file COPYING.  If not, write to
the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

*/

/*

=head1 NAME

c_string - string object for c++

=head1 SYNOPSIS

  #include <c_string.h>
  ...
  c_string s1("this is some text"), s2("this is some more"), s3;

  s3 = s1 + s2;
  cout << "s3 is: " << s3 << endl;

=head1 DESCRIPTION

c_string is a dynamicly growable string object.  It can act
like a const char*, and provides a great deal of added 
functionality.

c_string's internal comparison functions depend on the C
function strcmp.

=head1 API REFERENCE

=head2 c_string( const char* buff = (const char*)NULL )

=head2 c_string( char ch, unsigned int rpt = 1 )

=head2 c_string( const c_string& that )

Constructors.  

The first is the default, it takes an optional
const char* that points to the buffer the c_string should
initialize itself with.

The second takes a single character and an optional repeat
count.  This creates a string that consists of the character
repeated multiple times.

The third is a copy constructor.  The constructed c_string
will be a copy of the given string object.

=head2 virtual ~c_string()

Destructor.  Cleans up any memory allocated by the string object.

=head2 virtual void empty( void )

Blanks out the string.  

=head2 virtual const c_string& operator=( const c_string& right )

=head2 virtual const c_string& operator=( const char* buff )

=head2 virtual const c_string& operator=( char ch )

These are assignment operators for the following data types:
another c_string, a C style null terminated string, or a single
character.

=head2 virtual int operator!( void ) const

This can be used to test weather or not a c_string has a
length.
Returns true if the string has a length, otherwise returns false.

=head2 friend c_string operator+( const c_string& left, const c_string& right )

Friend function, addition operator.  Takes two c_string objects and returns
a temporary that is a concatenation of the two.

=head2 virtual const c_string& operator+=( const c_string& right )

=head2 virtual const c_string& operator+=( char ch )

=head2 virtual const c_string& operator+=( const char* buff )

These operators append the given data onto the end of the string 
object.

=head2 virtual int operator==( const c_string& right ) const

=head2 virtual int operator==( const char* buff ) const

=head2 virtual int operator==( char ch ) const

These equality operators test the contents of the c_string object
against the given data.

=head2 virtual int operator!=( const c_string& right ) const

=head2 virtual int operator!=( const char* buff ) const

=head2 virtual int operator!=( char ch ) const

Not equal operators for the given data types.

=head2 virtual int operator<( const c_string& right ) const

=head2 virtual int operator<( const char* buff ) const

=head2 virtual int operator<( char ch ) const

Less than operator for c_string, and the given types

=head2 virtual int operator<=( const c_string& right ) const

=head2 virtual int operator<=( const char* buff ) const

=head2 virtual int operator<=( char ch ) const

Less than or equal to operator for c_string, and the given types

=head2 virtual int operator>( const c_string& right ) const

=head2 virtual int operator>( const char* buff ) const

=head2 virtual int operator>( char ch ) const

Greater than operator for c_string, and the given types

=head2 virtual int operator>=( const c_string& right ) const

=head2 virtual int operator>=( const char* buff ) const

=head2 virtual int operator>=( char ch ) const

Greater than or euqal to operator for c_string, and the given types

=head2 virtual int ltrim( void )

Left trims whitespace (spaces and tabs) from the c_string.

=head2 virtual int rtrim(void)

Right trims whitespace (spaces and tabs) from the c_string.

=head2 virtual int eat_white( void )

Removes whitepsace that is from the string, by calling
ltrim(), and then rtrim().

=head2 virtual int get_length( void ) const

Returns' the length of the c_string object.

=head2 virtual void get_buff( char *buff ) const

Copies the string into the supplied buffer.

=head2 virtual void set_buff( char* buff )

Copies the given buffer into the c_string's buffer.

=head2 virtual int find( char ch ) const

=head2 virtual int find( const char* str ) const

Returns the index of the first occurrance of the given data 
in the string object.
If the data cannot be found, it returns -1.

=head2 virtual int rfind( char ch ) const

=head2 virtual int rfind( const char* str ) const

Returns the index of the first occurrance (from the right)
of the given data in the string object.  
If the data cannot be found, it returns -1.

=head2 virtual char& operator[]( int index )

The index operator returns a char reference, that references
the character at the given index.  The returned reference is
suitable as both an lvalue and an rvalue.

=head2 virtual int left( int len, c_string& lft ) const

=head2 virtual int right( int len, c_string& rt ) const

=head2 virtual int mid( int start, int len, c_string& md ) const

These three functions can be used to retreive subsections
of the string ojbect.  
They return true on success, otherwise they return false.

=head2 virtual operator const char*(void) const

This casting operator allows the c_string object to
be used in function calls and contexts where a const char*
(C style string) is expected.  

=head2 virtual const c_string reverse( void ) const

Returns a reversed version of the string.

=head2 virtual const c_string upper( void ) const

Returns an uppercase version of the string.

=head2 virtual const c_string lower( void ) const

Returns a lowercase version of the string.

=head2 static int strcmp( const char* buff1, const char* buff2 )

This is a safer strcmp, used internaly by the c_string object
to perform comparisons.  On some platforms, strcmp can not 
handle NULL as one or both of the arguments.  This version
can.

=head2 friend ostream& operator<<( ostream& o, const c_string& s )

Leftshift operator for an ostream.  This allows a c_string to be
shifted to cout, or an fstream object.  It provides the same
behavior as the const char* version of this operator provided by
the ostream objects.

=head2 void dump(void)

This is a diagnostic function used to display the string object's
internal state to cout.

=head2 int consists_of( const char ch ) const

Returns true if the string consists entirely of the 
given character, otherwise it returns false.

=head1 BUGS

=head1 AUTHORS

Kyle R. Burton mortis@voicenet.com

=head1 SEE ALSO

=cut

*/

/////////////////////////////////////////////////////////////////////
// file:  c_string.h
//
// description:  declaration of basic c_string class.
//
// created:  Sat 10/25/1997 14:34:17.26 Kyle R. Burton
//  
// Wed Dec 24 09:25:43 1997 KRB - changed:
//    virtual operator const char*(void);
//  to:
//    virtual operator const char*(void) const;
//
#if !defined( _C_STRING_H )
#define _C_STRING_H 

#include <iostream.h>
#include <stdio.h>
#include <string.h>

#include "stdinc.h"

#define SAFE_STRLEN(a)   ((a==NULL)?0:(a[0]=='\0')?0:strlen(a))

//
// class:  c_string
//
// description:
//
class c_string
{
public:
  //
  // c_string( const char* buff = (const char*)NULL )
  //
  // default constructor, optionaly accepts C style string
  // as an initializer.
  //
  c_string( const char* buff = (const char*)NULL );

  // 
  // c_string( char ch, unsgined int rpt = 1 )
  //
  // constrcutor that accepts single character [ch], and a repeat
  // count [rpt] as an initializer.
  //
  c_string( char ch, unsigned int rpt = 1 );

  //
  // c_string( const c_string& that )
  //
  // copy constructor.
  //
  c_string( const c_string& that );

  //
  // virtual ~c_string()
  //
  // destructor
  //
  virtual ~c_string();

  //
  // virtual void empty( void )
  //
  // sets internal buffer to empty state.
  //
  virtual void empty( void );

  //
  // virtual const c_string& operator=( const c_string& right )
  // virtual const c_string& operator=( const char* buff )
  // virtual const c_string& operator=( char ch )
  //
  // various assingment operators
  //
  virtual const c_string& operator=( const c_string& right );
  virtual const c_string& operator=( const char* buff );
  virtual const c_string& operator=( char ch );

  //
  // virtual int operator!( void)
  //
  // equivalent to: (0 == get_length())
  //
  virtual int operator!( void ) const;

  //
  // friend c_string operator+( const c_string& left, const c_string& right )
  //
  // friend addition operator.
  //
  friend c_string operator+( const c_string& left, const c_string& right );

  //
  // virtual const c_string& operator+=( const c_string& right )
  // virtual const c_string& operator+=( char ch )
  // virtual const c_string& operator+=( const char* buff )
  //
  // various implementations of additive assignment [+=]
  //
  virtual const c_string& operator+=( const c_string& right );
  virtual const c_string& operator+=( char ch );
  virtual const c_string& operator+=( const char* buff );

  //
  // virtual int operator==( const c_string& right ) const
  // virtual int operator==( const char* buff ) const
  // virtual int operator==( char ch ) const
  //
  // various implementations of equality operator
  //
  virtual int operator==( const c_string& right ) const;
  virtual int operator==( const char* buff ) const;
  virtual int operator==( char ch ) const;

  virtual int operator!=( const c_string& right ) const;
  virtual int operator!=( const char* buff ) const;
  virtual int operator!=( char ch ) const;

  virtual int operator<( const c_string& right ) const;
  virtual int operator<( const char* buff ) const;
  virtual int operator<( char ch ) const;

  virtual int operator<=( const c_string& right ) const;
  virtual int operator<=( const char* buff ) const;
  virtual int operator<=( char ch ) const;

  virtual int operator>( const c_string& right ) const;
  virtual int operator>( const char* buff ) const;
  virtual int operator>( char ch ) const;

  virtual int operator>=( const c_string& right ) const;
  virtual int operator>=( const char* buff ) const;
  virtual int operator>=( char ch ) const;


  //
  // ltrim removes whitespace (' ', and '\t') from the 
  // left side of the string, while rtrim does the same
  // for the right side of the string.
  //
  virtual int ltrim( void );
  virtual int rtrim(void);

  //
  // removes leading, and trailing whitespace characters: ' ' and '\t'
  //
  virtual int eat_white( void );

  //
  // virtual int get_length( void ) const
  //
  // returns lengtht of the stored string
  //
  virtual int get_length( void ) const;

  //
  // virtual void get_buff( char *buff ) const
  //
  // creates a copy of the internal buffer at buff.
  //
  virtual void get_buff( char *buff ) const;

  // 
  // virtual void set_buff( char* buff )
  //
  // copies buff into the internal buffer.
  //
  virtual void set_buff( char* buff );
  
  //
  // virtual int find( char ch ) const
  // virtual int find( const char* str ) const
  //
  // returns the index of the requested item, or -1 if not found
  //
  virtual int find( char ch ) const;
  virtual int find( const char* str ) const;
  
  //
  // virtual int rfind( char ch ) const
  // virtual int rfind( const char* str ) const
  //
  // identical to find() except that the search is performed in reverse
  //
  virtual int rfind( char ch ) const;
  virtual int rfind( const char* str ) const;
  
  //
  // virtual char& operator[]( int index )
  //
  // returns refrence to the character at index - throws exception if
  // index is invalid for the string.
  //
  virtual char& operator[]( int index );
  
  //
  // virtual int left( int len, c_string& lft ) const
  //
  // sets lft to the left len characters of the string, returns TRUE on
  // success, and FALSE on failure
  //
  virtual int left( int len, c_string& lft ) const;
  
  //
  // virtual int right( int len, c_string& rt ) const
  //
  // sets rt to the right len characters of the string, returns TRUE on
  // success, and FALSE on failure
  //
  virtual int right( int len, c_string& rt ) const;
  
  //
  // virtual int mid( int start, int len, c_string& md ) const
  //
  // sets md to len characters in the string starting at start
  // returns TRUE on success, FALSE on failure
  //
  virtual int mid( int start, int len, c_string& md ) const;
  
  //
  // virtual operator const char*(void) const;
  //
  // provides direct access to the internal buffer
  // also allows c_string to be used in place of a C style string
  //
  virtual operator const char*(void) const;

  //
  // virtual const c_string reverse( void ) const;
  //
  // returns a constant copy of the string with the buffer reversed
  //
  virtual const c_string reverse( void ) const;
  
  // 
  // virtual const c_string upper( void ) const;
  // virtual const c_string lower( void ) const;
  //
  // upper() returns a constant copy of the buffer with all
  // the alphabetic characters in uppercase.
  //
  // lower() reutrns a constant copy of the buffer with all
  // the alphabetic characters in lowercase.
  //
  virtual const c_string upper( void ) const;
  virtual const c_string lower( void ) const;

  //
  // string compare function that works a bit different than
  // the standard strcmp function -- mostly used internaly
  // by c_string
  //
  static int strcmp( const char* buff1, const char* buff2 ) {
    if( (const char*)NULL == buff1 && (const char*)NULL == buff2 ) {
      return 0;
    }

    if( (const char*)NULL == buff1 ) {
      return 1;
    }

    if( (const char*)NULL == buff2 ) {
      return -1;
    }

    if( buff1[0] == '\0' && buff2[0] == '\0' ) {
      return 0;
    }

    if( buff1[0] == '\0' ) {
      return 1;
    }

    if( buff2[0] == '\0' ) {
      return -1;
    }

    return ::strcmp( buff1, buff2 );
  };

  //
  // operator for left shifting into an ostream...
  // eg:   cout << c_string("this") << endl;
  //
  friend ostream& operator<<( ostream& o, const c_string& s ) {
    o << (const char*)s.m_buff;
    return o;
  };

  //
  // diagnostic dump function...
  //
  void dump(void) {
    char addr[32];
    sprintf( addr, "0x%X", m_buff );
    cout << __FILE__ << "(" << __LINE__ << ") [c_string] buff[" << addr << "]:        " 
      << m_buff << endl;
  };

  //
  // tests weather or not a string consits completely of a single character
  //
  int consists_of( const char ch ) const {
    char *p = m_buff;
    if( NULL == m_buff ) return 0;
    while( '\0' != *p ) {
      if( ch == *p ) p++;
      else return 0;
    }

    return 1;
  };

private:
  char  *m_buff;
  int   m_len;

  // 
  //   void copy( const char* buff )
  // 
  // copies buff into the internal buffer, allocating new memory if 
  // necessary
  //
  void copy( const char* buff );
};


#endif // !defined( _C_STRING_H )
