#ifdef IGNORE_ME
  #define ident "@(#) c_timestamp.h (/sccs/ri_util/include/s.c_timestamp.h) 1.6 02/24/99 14:20:44";
#endif

/*  c_timestamp.h
  Copyright 1997 Kyle R. Burton

c_timestamp.h is free software; you can redistribute it and/or modify
it under the tearms of the GNU General Public License as published by 
the Free Software Foundation; either version 2, or (at your option)
any later version.

c_timestamp.h is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with c_timestamp.h; see the file COPYING.  If not, write to
the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

*/

/*

=head1 NAME

c_timestamp - convienience class to get the current time

=head1 SYNOPSIS

  #include <c_timestamp.h>
  ...
  cout << "The current time is: " << (c_string)c_timestamp() << endl;

=head1 DESCRIPTION

This class can be used as a convienient way to retreive the current 
system time, in a consistient format.  It was orignally intended for
logging purposes.

=head1 API REFERENCE

=head2 operator c_string( void ) const

c_string casting operator -- returns a temporary c_string 
that contains the sprintf'd time value.

=head1 BUGS

On some platforms (such as HP-UX), some of the time functions were
not thread safe.  Thusly we had to remove the calls that formatted
the time structure into a human readable format.

=head1 AUTHORS

Kyle R. Burton mortis@voicenet.com

=head1 SEE ALSO

=cut
*/

#if !defined( _C_TIMESTAMP_H )
#define _C_TIMESTAMP_H 

#include <time.h>
#include <c_string.h>

class c_timestamp
{
public:
  operator const char*( void ) const {
    static char buff[128];
    time_t t;
    time(&t);
    // sprintf(const_cast<char*>(buff),"%08x",t);
    //sprintf((char*)(buff),"%08x",t);
    //return(buff);
    char *rv = ctime(&t);
    char *p = strrchr(rv,'\n'); if( NULL != p ) *p = '\0';
          p = strrchr(rv,'\r'); if( NULL != p ) *p = '\0';
    return rv;
  };
private:
};



#define NOW  "[" << (const char*)c_timestamp() << "] " 

#endif // !defined( _C_TIMESTAMP_H )

