#ifdef IGNORE_ME
  #define ident "@(#) date.h (/sccs/ri_util/include/s.date.h) 1.2 02/24/99 14:20:44";
#endif

  /*
 *
 * file: date.h
 *
 * description: header file for standard date functions (extensions to the std C library)
 *
 * Created: 05/27/97 Kyle Burton, Ksenia Kaliaguina
 *
 * 
 */

#if !defined ( _DATE_H )

#if !defined( TRUE )
#define TRUE 1
#endif /* !defined( TRUE ) */

#if !defined( FALSE )
#define FALSE 0
#endif /* !defined( FALSE ) */

#if !defined( NULL )
#define NULL (void*)0
#endif

/* defines for months, starting at 0... */
#define JAN  0
#define FEB  1 
#define MAR  2
#define APR  3
#define MAY  4
#define JUN  5
#define JUL  6
#define AUG  7
#define SEP  8
#define OCT  9
#define NOV  10
#define DEC  11

/* defines for days, starting at 0... */
#define SUN  0
#define MON  1
#define TUE  2
#define WED  3
#define THU  4
#define FRI  5
#define SAT  6

/* defines for converting to and from the Oracle datatype */
#define CENTURY_OFFSET  0
#define YEAR_OFFSET  1
#define MON_OFFSET  2
#define DAY_OFFSET  3
#define HOUR_OFFSET  4
#define MIN_OFFSET  5
#define SEC_OFFSET  6

#if defined ( __cplusplus )
  extern "C" {
#endif /* __cplusplus */

    /*
     * NAME
     *   ValidDate
     *
     * SYNOPSIS
     *   #include <date.h>
     *
     * DESCRIPTION
     *   Arguments:
     *    nYear         the year, see LeapYear for allowable formats
     *    nMonth        the month, 1-12
     *    nDay          day, 1-31
     *
     *   A date is considered valid if it meets the following criteria:
     *   - The month is greater than 0 and less than 13
     *   - The day is valid for the month sepcified (i.e. if 02/30/97 is invalid)
     *   - Leap years may contain 29 days in Feburary, the function LeapYear is used
     *   to determine if a year is a leap year.
     *
     * RETURNS
     *   Returns 1 if a date is valid, and 0 if it is not.
     *
     *
     * Created: 05/27/97 Kyle R. Burton, Ksenia Kaliaguina
     *
     */
    int ValidDate( int nYear, int nMonth, int nDay );

    /* Function: LeapYear
     * NAME
     *   LeapYear
     *
     * SYNOPSIS
     *   #include <date.h>
     *  
     *   int LeapYear( int nYear );
     *
     * DESCRIPTION
     *   LeapYear returns non-zero if the year sepcified is a leap year.
     *   A file is considered to be a leap year if it is evenly divisible
     *   by four.  
     *
     * RETURNS
     *   Non-zero if nYear is a leap year, zero if it is not.
     *
     * NOTES
     *   The algorithm used will only be valid for years falling within the 
     *   20th and 21st centuries.
     *
     * CREATED: 05/27/97 Kyle R. Burton, Ksenia Kaliaguina
     *
     */
    int LeapYear( int nYear );


    /*
     * NAME
     *   DateToChar
     *
     * SYNOPSIS
     *   #include <date.h>
     *
     * DESCRIPTION
     *   Arguments:
     *    char* fmt  a string that specifies the format
     *    char* s  a 7 byte array that contains the date to format
     *    char* target  a buffer to receive the formatted date
     *
     *   Any unrecognized characters are copied directly into the target 
     *   string.  Valid format character sequences are:
     *
     *      DD   2 digit day of month - zero padded
     *      D    non-padded version of DD
     *      HH   2 digit hour of the day - zero padded, 24hr format
     *      H    non-padded version of HH
     *      MI   2 digit minute of hour - zero padded
     *      Mi   non-padded version of MI
     *      MM   2 digit month of year - zero padded
     *      M    non-padded version of MM
     *      SS   2 digit second of minute - zero padded
     *      S    non-padded version of SS
     *      YYYY Each of these produces a year format,
     *      YYY  with the first form giving the entire 4 digit year,
     *      YY   and each subsequent format giving 1 less digit from the
     *      Y    year value - for example, from the year 2003:
     *           YYYY=2003; YYY=003; YY=03; Y=3.
     *      yyyy These formats are identical to the uppercase versions
     *      yyy  except for the attribute that the uppercase versions
     *      yy   are zero padded out to their length, while these are 
     *      y    not.
     *           For Example, from the year 2003:
     *           yyyy=2003; yyy=3; yy=3; y=3.
     *
     * RETURNS
     *   Returns TRUE on succes, otherwise FALSE.
     *
     *
     * Created: 07/12/97 Kyle R. Burton
     *
     */
int DateToChar( char* fmt, char* s, char* target );

    /*
     * NAME
     *   CharToDate
     *
     * SYNOPSIS
     *   #include <date.h>
     *
     * DESCRIPTION
     *   Arguments:
     *    char* fmt  a string that specifies the format to use when parsing s
     *    char* s  the string to parse
     *    char* target  a 7 byte buffer to hold the date parsed
     *
     *   For more information regarding the fmt string, see DateToChar(3).
     *
     * RETURNS
     *   Returns TRUE upon success, otherwise it returns FALSE.
     *
     *
     * Created: 07/12/97 Kyle R. Burton
     *
     */
int ToCharDate( char* fmt, char* s, char* target );

#if defined( __cplusplus )
  }
#endif /* __cplusplus */

#endif /* _DATE_H */
