/*
 * file:    std_init.h
 *
 * description:  declaration of all init related functions/objects
 *
 * revision list:
 *   Fri May  8 11:19:54 EDT 1998 KRB - created file.
 *
 *
 *
 */

/*

=head1 NAME

std_init - standard initialization routine(s)

=head1 SYNOPSIS

  #include <std_init.h>
  ...
  c_hash<c_string,c_string> settings;
  init_configure("/etc/inetd.conf",settings);
  c_string tmp;
  settings.find(c_string("telnet"),tmp);
  cout << "telnet: " << tmp << endl;

=head1 DESCRIPTION

Standard module for parsing configuration files.  

The function takes two parameters:  a file name,
and a reference to a hash table.

Lines will be broken up into two parts.  A hash (#) 
character, and all characters between it and the end 
of the line are first stripped.  Any leading whitespace 
is also stripped.  The first block of non-whitespace 
text is extraced to become the key.  All whitespace 
between the key and the next non-whitespace character
is ignored, and the remainder of the line is taken as 
the value.

The key value paris are then stored in the hash table
passed as the second argument.

=head1 BUGS

Currently ther are no provisions for using a hash (#) in either the
key or the value.

=head1 AUTHORS

Kyle R. Burton mortis@voicenet.com

=head1 SEE ALSO

L<getopts>

*/

#ifdef IGNORE_ME
  #define ident "@(#) std_init.h (/sccs/ri_util/include/s.std_init.h) 1.4 02/24/99 14:20:45";
#endif

#if !defined( _STD_INIT )
#define _STD_INIT 

#include <c_string.h>
#include <c_hash.h>

/**************************************************************************************
 * init_configure takes a file name, and a refrence to a hash table
 *   it parses the file as a standard unix resource file (hash for single line comments,
 *   with key/values seperated by whitespace.
 */
int init_configure( const char* file_name, c_hash<c_string,c_string>& settings );

#endif // !defined( _STD_INIT )









