#ifdef IGNORE_ME
  #define ident "@(#) str_ext.h (/sccs/ri_util/include/s.str_ext.h) 1.3 02/24/99 14:20:44";
#endif

  /* 
 * file: str_ext.h
 *
 * Created: 06/11/97 Kyle R. Burton
 *
 * Revision List:
 *
 */

/*

=head1 NAME

str_ext - string extension functions

=head1 SYNOPSIS

	#include <str_ext.h>
  ...

=head1 DESCRIPTION

This module provides string routines not generaly found
on ANSI or POSIX systems.

=head1 API REFERENCE 

=head2 char* strlpad( char ch, unsigned int uTotalLen, char* szText )

Left pads a string with the given character untill the total length 
of the string reaches uTotalLen.

=head2 char* strrpad( char ch, unsigned int uTotalLen, char* szText )

Right pads a string with the given character untill the total length 
of the string reaches uTotalLen.

=head2 char* strrepl( char *s, char *f, char *t )

strrepl will replace any  occurance of the string f with the 
string t in the string s.  s must be large enough to contain
the resultant  string.   strrepl returns  a  pointer to s to
indicate  success, NULL is returned in the event that memory
can not be allocated  for a  temporary  buffer.  s, f, and t
must be NULL terminated strings.

=head2 void* memrepl( void *s, unsigned int s_len, void *f, unsigned int f_len, void *t, unsigned int t_len )

memrepl will replace any  occurance of the string f with the 
string t in the string s.  s must be large enough to contain
the resultant  string.   memrepl returns  a  pointer to s to
indicate  success, NULL is returned in the event that memory
can not be allocated  for a  temporary  buffer.

=head2 char* strupper( char* s )

Uppercases the given string, character by character.

=head1 BUGS

No known bugs.

=head1 AUTHORS

Kyle R. Burton mortis@voicenet.com

=head1 SEE ALSO

L<c_string>

=cut

*/

#if !defined( _STR_EXT_H )
#define _STR_EXT_H

/* 
 * NAME
 *  strlpad, strrpad
 *
 * SYNOPSIS
 *      #include <str_ext.h>
 *
 *  char* strlpad( char ch, unsigned int uTotalLen, char* szText );
 *  char* strrpad( char ch, unsigned int uTotalLen, char* szText );
 *
 * DESCRIPTION
 *  strlpad    strlpad will left pad szText with ch untill
 *      it reaches uTotalLengh.  szText  must be at
 *      least  uTotalLen + 1 characters in  length.
 *  strrpad    Right  pads  szText  with ch untill  szText
 *      reaches uTotalLength.
 *
 */

char* strlpad( char ch, unsigned int uTotalLen, char* szText );
char* strrpad( char ch, unsigned int uTotalLen, char* szText );

/* 
 * NAME
 *      strrepl
 *
 * SYNOPSIS
 *      #include <str_ext.h>
 *
 *      char* strrepl( char *s, char *f, char *t );
 *
 * DESCRIPTION
 *      strrepl will replace any  occurance of the string f with the 
 *  string t in the string s.  s must be large enough to contain
 *  the resultant  string.   strrepl returns  a  pointer to s to
 *      indicate  success, NULL is returned in the event that memory
 *      can not be allocated  for a  temporary  buffer.  s, f, and t
 *      must be NULL terminated strings.
 *
 */
char* strrepl( char *s, char *f, char *t );

/* 
 * NAME
 *      memrepl
 *
 * SYNOPSIS
 *      #include <str_ext.h>
 *
 *      void* memrepl( void *s, unsigned int s_len, 
 *                     void *f, unsigned int f_len,
 *                     void *t, unsigned int t_len );
 *
 * DESCRIPTION
 *      memrepl will replace any  occurance of the string f with the 
 *  string t in the string s.  s must be large enough to contain
 *  the resultant  string.   memrepl returns  a  pointer to s to
 *      indicate  success, NULL is returned in the event that memory
 *      can not be allocated  for a  temporary  buffer.
 *
 */
void* memrepl( void *s, unsigned int s_len, 
    void *f, unsigned int f_len, 
    void *t, unsigned int t_len );


char* strupper( char* s );

#endif /* _STR_EXT_H */

