#ifdef IGNORE_ME
  #define ident "@(#) str_map.h (/sccs/ri_util/include/s.str_map.h) 1.2 02/24/99 14:20:44";
#endif

  //////////////////////////////////////////////////////////////////////////////////
// file: str_map.h
//
// description: declaration of the string map class, and supporting classes.
//
// created: 06/28/97 Kyle R. Burton
//
// revision list:
//
#if !defined( _STR_MAP_H )
#define _STR_MAP_H

#include "c_collection.h"

class c_map_exception

template<class T>
class c_map_iter
  : virtual public c_collection_iter<T>
{
public:
  friend class str_map;
private:
  void init( void ) {

  };
};

#if 0
//////////////////////////////////////////////////////////////////////////////////
// class str_map_node
//
// binary tree node used by the str_map class
//
template<class T>
class str_map_node 
  : virtual public c_collection 
{
public:
  char*    key;  // identification key
  void*    data;  // data that is mapped to key

  str_map_node*  parent;  // node's parent
  str_map_node*  left;  // note's left child
  str_map_node*  right;  // note's right child

  void init( void ) { // sets default state
    key  = (char*)NULL;
    data  = NULL;
    parent  = (str_map_node*)NULL;
    left  = (str_map_node*)NULL;
    right  = (str_map_node*)NULL;
  };

  void clear( void ); // sets invalid state

public:
  str_map_node( const char* k = (char*)NULL, void* d = NULL );
  ~str_map_node( );
};
#endif

//////////////////////////////////////////////////////////////////////////////////
// class str_map
//
// str_map's purpose is to provide an efficient way of storing key/value pairs, where
// values are accessed by a unique character string.
//
template<class T>
class str_map 
  : virtual public c_collection<T>
{
public:
  str_map( );
  str_map( str_map& that );
  ~str_map( );

  int get_count( void ) { return( count ); };

  // NAME
  //   str_map::add
  //
  // SYNOPSIS
  //   #include <str_map.h>
  // 
  //   int add( const char* k, void* d );
  //   int add( str_map_node* n );
  //
  // DESCRIPTION
  //   These two functions are used to insert a new pair into
  //   the map.  The first  version  will  allocate new space
  //   for a str_map_node object, including a copy of d.  The 
  //   first  version  will throw a  c_exception if the memory
  //   allocation fails.  
  // 
  // RETURN
  //   add returns  non-zero  if  the node  can be added, and 
  //   zero  if  the node cannot.   add will fail if the node
  //   already  exists.   add  (first version)  will  throw a 
  //   c_exception  if  it  is  unable to  allocate memory for 
  //   the new node.
  //
  int add( const char* k, void* d );
  int add( str_map_node* n );

  // NAME
  //   str_map::remove
  //
  // SYNOPSIS
  //   #include <str_map.h>
  // 
  //   int remove( str_map_iter* pos );
  //
  // DESCRIPTION
  //   remove will take the node refrenced by pos, and remove 
  //   it from the  tree.  The tree will be rearranged in the
  //   following manner:  
  //    - pos refrences the top of the tree (parent is NULL):
  //         the left  child  node  becomes  the top, and the 
  //         right  child  node  is  placed at the end of the 
  //         left node's sub-tree.
  //    - pos refreneces a leaf node:
  //         the node is removed, and it's parent is updated.
  //    - pos refrences a center node:
  //         the  node  is  replaced with  the  left child of 
  //         itself, it's right child  is  placed at  the end
  //         of the left node's sub-tree.
  //   the node refrenced by pos is freed in every case.
  //
  // RETURN
  //   remove will return  non-zero  if the remove it is able 
  //   to complete the remove, and zero if it cannot.  remove
  //   will fail if pos is invalid.
  //
  int remove( str_map_iter* pos );

  // NAME
  //   str_map::head
  //
  // SYNOPSIS
  //   #include <str_map.h>
  // 
  //   str_map_iter* head( str_map_iter* pos );
  //
  // DESCRIPTION
  //   head returns the top  most node in a tree.  If pos is 
  //   NULL, a new str_map_iter is allocated, and set to the
  //   top of the tree the object represents.  If pos is not
  //   NULL, the  tree  it refrences  is  travsersed upwards 
  //   untill the top is reached.
  //
  // RETURN
  //   head returns  a  poitner to a  str_map_iter object if 
  //   it is successful, and NULL on failure. 
  //
  // NOTES
  //   the calling process  is  responsible for  freeing the
  //   allocated str_map_iter  object if NULL was passed for
  //   pos.
  //
  str_map_iter* head( str_map_iter* pos = (str_map_iter*)NULL );

  // NAME
  //   str_map::greatest
  //
  // SYNOPSIS
  //   #include <str_map.h>
  // 
  //   str_map_iter* greatest( str_map_iter* pos );
  //
  // DESCRIPTION
  //   This function will return the  right most  (greatest)
  //   element from the tree.   If NULL is passed for pos, a
  //   new  str_map_iter will  be  allocated.  If pos is not 
  //   NULL,  then  the  tree  that  pos  refrences  will be 
  //   traversed.
  //
  // RETURN
  //   pos  or a  new  str_map_iter are returned on success,
  //   NULL is returned on failure.
  //
  // NOTES
  //   The calling  process  is  responsible for freeing the
  //   new str_map_iter  if NULL  is passed  for pos.   If a
  //   valid pos is passed, the tree will _NOT_ be traversed
  //   from the top, it will be  traversed from the node pos
  //   refrences down.
  //
  str_map_iter* greatest( str_map_iter* pos = (str_map_iter*)NULL );

  // NAME
  //   str_map::least
  //
  // SYNOPSIS
  //   #include <str_map.h>
  // 
  //   str_map_iter* least( str_map_iter* pos );
  //
  // DESCRIPTION
  //   least  returns  the  left most  node (least) in the 
  //   tree. If NULL is passed for pos, a new str_map_iter
  //   will be allocated.  If  pos is not  NULL,  then the 
  //   tree that pos refrences will be traversed.
  //
  // RETURN
  //   pos or a new  str_map_iter are returned on success,
  //   NULL  is returned on  failure.   If a valid  pos is 
  //   passed, the tree will  _NOT_  be traversed from the 
  //   top,  it  will  be  traversed  from  the  node  pos 
  //   refrences down.
  //
  // NOTES
  //   The calling  process  is  responsible for freeing the
  //   new str_map_iter if NULL is passed for pos.
  //
  str_map_iter* least( str_map_iter* pos = (str_map_iter*)NULL );

  // NAME
  //   str_map::next
  //
  // SYNOPSIS
  //   #include <str_map.h>
  // 
  //   str_map_iter* next( str_map_iter* pos );
  //
  // DESCRIPTION
  //   If NULL is passed for pos, then the left most node
  //   in the tree is returned, otherwise, the next node
  //   in the tree refrenced by pos is returned.  If there
  //   are no more nodes in the tree, NULL is returned.  
  //
  // RETURN
  //   pos, or a new str_map_iter object on success, and
  //   NULL on failure.
  //
  // NOTES
  //   If NULL is passed for pos, then this is functionaly
  //   equivalent to str_map::least.
  //
  str_map_iter* next( str_map_iter* pos = (str_map_iter*)NULL );

  // NAME
  //   str_map::prev
  //
  // SYNOPSIS
  //   #include <str_map.h>
  // 
  //   str_map_iter* prev( str_map_iter* pos );
  //
  // DESCRIPTION
  //   If NULL is passed for pos, then the right most node
  //   in the tree is returned, otherwise, the previous node
  //   in the tree refrenced by pos is returned.  If there
  //   are no more nodes in the tree, NULL is returned.  
  //
  // RETURN
  //   pos, or a new str_map_iter object on success, and
  //   NULL on failure.
  //
  // NOTES
  //   If NULL is passed for pos, then this is functionaly
  //   equivalent to str_map::greatest.
  //
  str_map_iter* prev( str_map_iter* pos  = (str_map_iter*)NULL );

  // NAME
  //   str_map::find
  //
  // SYNOPSIS
  //   #include <str_map.h>
  // 
  //   str_map_iter* find( const char* k, str_map_iter* pos );
  //
  // DESCRIPTION
  //   find searches for the node who's key matches k.  If
  //   NULL is passed for pos, then the current tree is
  //   searched, and a new str_map_iter is allocated.  If
  //   pos is valid, then the tree that pos refrences is 
  //   searched.
  //
  // RETURN
  //   pos, or a new str_map_iter if a node that matches k is
  //   found, and NULL on failure.
  //
  // NOTES
  //
  str_map_iter* find( const char* k, str_map_iter* pos = (str_map_iter*)NULL );

  friend class c_str_map_iter;
private:
  str_map_node*     top;
  int      count;

  void init( void ) {
    top = (str_map_node*)NULL;
    count = 0;
  };
};

#endif // _STR_MAP_H
